import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;

import com.moyosoft.exchange.*;
import com.moyosoft.exchange.attachment.*;
import com.moyosoft.exchange.item.*;

import java.io.*;
import java.util.ArrayList;
import java.util.List;

public class ExchangeAttachmentsPanel extends JPanel
{
	private JButton mButtonAdd = new JButton("Add");
	private JButton mButtonRemove = new JButton("Remove");
	private JButton mButtonSaveAs = new JButton("Save as");
	private JList mAttachmentList = new JList();
	private AttachmentListModel mListModel = new AttachmentListModel();
	private ExchangeItem mItem;
	
	public ExchangeAttachmentsPanel()
	{
		super();
		init();
	}
	
	protected void init()
	{
		createPanel();
	}

	protected JPanel createButtonPanel()
	{
		JPanel buttonPanel = new JPanel(new BorderLayout());
		JPanel panel = new JPanel(new GridBagLayout());
		
		GridBagConstraints labelConstraints = new GridBagConstraints();
		labelConstraints.insets = new Insets(0, 5, 2, 5);
		labelConstraints.anchor = GridBagConstraints.WEST;
		
		GridBagConstraints fieldConstraints = new GridBagConstraints();
		fieldConstraints.fill = GridBagConstraints.HORIZONTAL;
		fieldConstraints.weightx = 1;
		
		// 0,0
		labelConstraints.gridx = 0;
		labelConstraints.gridy = 0;
		panel.add(mButtonAdd, labelConstraints);

		// 0,1
		labelConstraints.gridx = 0;
		labelConstraints.gridy = 1;
		panel.add(mButtonRemove, labelConstraints);

		// 0,2
		labelConstraints.gridx = 0;
		labelConstraints.gridy = 2;
		panel.add(mButtonSaveAs, labelConstraints);
		
		buttonPanel.add(BorderLayout.NORTH, panel);
		
		return buttonPanel;
	}
	
	protected void createPanel()
	{
		// set the model
		mAttachmentList.setModel(mListModel);
		
		mButtonRemove.setEnabled(false);
		mButtonSaveAs.setEnabled(false);
		
		JPanel buttonPanel = createButtonPanel();
		JScrollPane scrollPane = new JScrollPane(mAttachmentList);
		
		setLayout(new BorderLayout());
		add(BorderLayout.CENTER, scrollPane);
		add(BorderLayout.EAST, buttonPanel);
		
		// listeners
		mButtonAdd.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent event)
			{
				addPressed();
			}
		});
		mButtonRemove.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent event)
			{
				removePressed();
			}
		});
		mButtonSaveAs.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent event)
			{
				saveAsPressed();
			}
		});
		mAttachmentList.addListSelectionListener(new ListSelectionListener()
		{
			public void valueChanged(ListSelectionEvent listSelectionEvent)
			{
				listSelectionChanged();
			}
		});
	}

	private void showError(Exception ex)
	{
		ex.printStackTrace();
		
		if(ex instanceof ExchangeServiceException)
		{
			System.out.println(((ExchangeServiceException) ex).getDetails());
		}
		
		JOptionPane.showMessageDialog(
				this,
				ex.getMessage(),
				"Error",
				JOptionPane.ERROR_MESSAGE
				);
	}
	
	protected void addPressed()
	{
		if(mItem == null)
		{
			return;
		}
		
		JFileChooser chooser = new JFileChooser();
		chooser.showOpenDialog(getTopLevelAncestor());
		
		File file = chooser.getSelectedFile();
		
		if(file != null)
		{
			try
			{
				mItem.getAttachments().add(file);
				refresh();
			}
			catch(ExchangeServiceException ex)
			{
				showError(ex);
			}
			catch(IOException ex)
			{
				showError(ex);
			}
		}
	}
	
	protected void removePressed()
	{
		int row = mAttachmentList.getSelectedIndex();
		
		ExchangeAttachment attachment = mListModel.getAttachmentAt(row);
		
		try
		{
			if(JOptionPane.showConfirmDialog(
					this,
					"Are you sure?",
					"Delete attachment",
					JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION)
			{
				attachment.delete();
				refresh();
			}
		}
		catch(ExchangeServiceException ex)
		{
			showError(ex);
		}
	}
	
	protected void saveAsPressed()
	{
		int row = mAttachmentList.getSelectedIndex();
		
		ExchangeAttachment attachment = mListModel.getAttachmentAt(row);
		
		if(attachment != null)
		{
			try
			{
				JFileChooser chooser = new JFileChooser();
				
				chooser.setSelectedFile(new File(attachment.getName()));
				chooser.showSaveDialog(getTopLevelAncestor());
				
				File file = chooser.getSelectedFile();
				
				if(file != null)
				{
					attachment.writeTo(file);
				}
			}
			catch(ExchangeServiceException ex)
			{
				showError(ex);
			}
			catch(IOException ex)
			{
				showError(ex);
			}
		}
	}
	
	protected void listSelectionChanged()
	{
		int row = mAttachmentList.getSelectedIndex();
		mButtonRemove.setEnabled(row >= 0);
		mButtonSaveAs.setEnabled(row >= 0);
	}
	
	protected Window getParentWindow()
	{
		Container c = this.getTopLevelAncestor();
		if(c instanceof Window)
		{
			return (Window) c;
		}
		return null;
	}
	
	private void refresh() throws ExchangeServiceException
	{
		load(mItem);
	}
	
	protected void load(ExchangeItem item) throws ExchangeServiceException
	{
		mItem = item;
		mListModel.clear();

		if(item != null && item.hasVisibleAttachments())
		{
			AttachmentsCollection attachments = item.getAttachments();
			
			if(attachments == null)
			{
				return;
			}
	
			for(ExchangeAttachment attachment : attachments)
			{
				mListModel.add(attachment);
			}
		}
	}
}

class AttachmentListModel extends AbstractListModel
{
	private List<AttachmentItem> mAttachmentList = new ArrayList<AttachmentItem>();
	
	public AttachmentListModel()
	{
	}
	
	public int getSize()
	{
		return mAttachmentList.size();
	}
	
	public Object getElementAt(int row)
	{
		AttachmentItem attachment = getAttachmentItemAt(row);
		
		if(attachment != null)
		{
			return attachment.getName();
		}
		
		return "";
	}
	
	public ExchangeAttachment getAttachmentAt(int row)
	{
		return getAttachmentItemAt(row).getAttachment();
	}
	
	protected AttachmentItem getAttachmentItemAt(int row)
	{
		return mAttachmentList.get(row);
	}
	
	public void clear()
	{
		int size = getSize();
		mAttachmentList.clear();
		fireIntervalRemoved(this, 0, size);
	}
	
	public void add(ExchangeAttachment attachment) throws ExchangeServiceException
	{
		if(attachment == null)
		{
			return;
		}
		
		int size = getSize();
		mAttachmentList.add(new AttachmentItem(attachment));
		fireIntervalAdded(this, size, size);
	}
	
	public void remove(int row)
	{
		mAttachmentList.remove(row);
		fireIntervalRemoved(this, row, row);
	}
	
	class AttachmentItem
	{
		private String mName;
		private ExchangeAttachment mItem;
		
		public AttachmentItem(ExchangeAttachment item) throws ExchangeServiceException
		{
			mItem = item;
			mName = item.getName();
		}
		
		public String getName()
		{
			return mName;
		}
		
		public ExchangeAttachment getAttachment()
		{
			return mItem;
		}
	}
}
