import java.awt.*;
import java.awt.event.*;

import javax.net.ssl.*;
import javax.swing.*;

import com.moyosoft.exchange.*;

public class ExchangeConnectionDialog extends JDialog
{
	private JButton mButtonCancel = new JButton("Cancel");
	private JButton mButtonOk = new JButton("Connect");

	private JTextField mHostname = new JTextField();
	private JTextField mUsername = new JTextField();
	private JTextField mPassword = new JTextField();
	private JTextField mDomain = new JTextField();
	
	private JCheckBox mUseHttps = new JCheckBox("Use secure connection (HTTPS)", true);
	private JCheckBox mIgnoreCertificateErrors = new JCheckBox("Ignore SSL certificate errors", false);
	private JCheckBox mSpecifyConnectionUrl = new JCheckBox("Use URL", false);
	private JTextField mConnectionUrl = new JTextField();
	private JPanel mMoreOptionsPanel = new JPanel(new GridBagLayout());
	private JButton mButtonMoreLess = new JButton("More \u25BC");
	
	private Exchange mExchange;
	
	public ExchangeConnectionDialog(Frame pParent)
	{
		super(pParent, "Connection to Exchange server", true);
		init();
	}

	public ExchangeConnectionDialog(Dialog pParent)
	{
		super(pParent, "Connection to Exchange server", true);
		init();
	}

	public ExchangeConnectionDialog()
	{
		super();
		this.setTitle("Connection to Exchange server");
		this.setModal(true);
		init();
	}

	private void init()
	{
		setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);

		mConnectionUrl.setEditable(false);
		mHostname.requestFocus();
		
		JPanel okCancelPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
		okCancelPanel.add(mButtonOk);
		okCancelPanel.add(mButtonCancel);

		JPanel moreLessPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
		moreLessPanel.add(mButtonMoreLess);

		JPanel buttonsPanel = new JPanel(new BorderLayout());
		buttonsPanel.add(BorderLayout.WEST, moreLessPanel);
		buttonsPanel.add(BorderLayout.EAST, okCancelPanel);

		int row = 0;
		addRow(mMoreOptionsPanel, new JLabel("Domain"), mDomain, row++);
		addRow(mMoreOptionsPanel, mUseHttps, null, row++);
		addRow(mMoreOptionsPanel, mIgnoreCertificateErrors, null, row++);
		addRow(mMoreOptionsPanel, mSpecifyConnectionUrl, mConnectionUrl, row++);
		mMoreOptionsPanel.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createEmptyBorder(10, 0, 5, 0), BorderFactory.createTitledBorder("Advanced options")));
		
		row = 0;
		JPanel contentPanel = new JPanel(new GridBagLayout());
		addRow(contentPanel, new JLabel("Hostname"), mHostname, row++);
		addRow(contentPanel, new JLabel("Username"), mUsername, row++);
		addRow(contentPanel, new JLabel("Password"), mPassword, row++);

		JPanel allOptionsPanel = new JPanel(new BorderLayout());
		allOptionsPanel.add(BorderLayout.NORTH, contentPanel);
		allOptionsPanel.add(BorderLayout.SOUTH, mMoreOptionsPanel);
		allOptionsPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        
		JPanel mainPanel = new JPanel(new BorderLayout());
		mainPanel.add(BorderLayout.NORTH, allOptionsPanel);
		mainPanel.add(BorderLayout.SOUTH, buttonsPanel);

		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(BorderLayout.CENTER, mainPanel);

		getRootPane().setDefaultButton(mButtonOk);
		
		mMoreOptionsPanel.setVisible(false);
		
		pack();
		setSize(320, getSize().height);
		centerScreen();

		mButtonCancel.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent event)
			{
				cancelPressed();
			}
		});

		mButtonOk.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent event)
			{
				okPressed();
			}
		});
		
		mButtonMoreLess.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent event)
			{
				if(mMoreOptionsPanel.isVisible())
				{
					mMoreOptionsPanel.setVisible(false);
					mButtonMoreLess.setText("More \u25BC");
					updateSize();
				}
				else
				{
					mMoreOptionsPanel.setVisible(true);
					mButtonMoreLess.setText("Less \u25B2");
					updateSize();
				}
			}
		});
		
		mSpecifyConnectionUrl.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent event)
			{
				mConnectionUrl.setEditable(mSpecifyConnectionUrl.isSelected());
			}
		});
	}

	private void addRow(JPanel contentPanel, JComponent component1, JComponent component2, int row)
	{
        GridBagConstraints c = new GridBagConstraints();

        c.fill = GridBagConstraints.HORIZONTAL;
        c.anchor = GridBagConstraints.NORTHWEST;

        if(component1 != null)
        {
	        c.gridx = 0;
	        c.gridy = row;
	        c.weightx = 0;
	        c.insets = component1 instanceof JCheckBox ? new Insets(0, 0, 0, 0) : new Insets(4, 5, 0, 5);
	        
	        if(component2 == null)
	        {
	        	c.gridwidth = 2;
	        }
	        
	        contentPanel.add(component1, c);
        }

        if(component2 != null)
        {
	        c.gridx = 1;
	        c.gridy = row;
	        c.weightx = 1;
	        c.insets = new Insets(2, 10, 0, 5);
	        contentPanel.add(component2, c);
        }
	}
	
	private void updateSize()
	{
		setSize(getSize().width, getPreferredSize().height);
		validate();
	}
	
	protected void centerScreen()
	{
		Dimension dim = getToolkit().getScreenSize();
		Rectangle abounds = getBounds();
		setLocation((dim.width - abounds.width) / 2,
				(dim.height - abounds.height) / 2);
		requestFocus();
	}

	protected void closeDialog()
	{
		dispose();
	}

	protected void cancelPressed()
	{
		closeDialog();
	}

	protected void okPressed()
	{
		if(mHostname.getText().trim().length() == 0)
		{
			JOptionPane.showMessageDialog(this, "Please specify a valid hostname", getTitle() + " - Error", JOptionPane.ERROR_MESSAGE);
			return;
		}
		
		try
		{
			Exchange.Settings.setTrustInvalidCertificate(mIgnoreCertificateErrors.isSelected());
			
			if(mMoreOptionsPanel.isVisible() && mSpecifyConnectionUrl.isSelected() && mConnectionUrl.getText().trim().length() > 0)
			{
				mExchange = new Exchange(
						mHostname.getText(),
						mUsername.getText(),
						mPassword.getText(),
						mDomain.getText().trim().length() == 0 ? null : mDomain.getText(),
						mConnectionUrl.getText()
						);
			}
			else
			{
				mExchange = new Exchange(
						mHostname.getText(),
						mUsername.getText(),
						mPassword.getText(),
						mDomain.getText().trim().length() == 0 ? null : mDomain.getText(),
						mUseHttps.isSelected()
						);
			}
		}
		catch(ExchangeServiceException ex)
		{
			if(!mIgnoreCertificateErrors.isSelected() && ex.getCause() instanceof SSLException)
			{
				if(JOptionPane.showConfirmDialog(this, "Unable to verify the SSL certificate. Connect anyway?", "Invalid SSL certificate", JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE) == JOptionPane.YES_OPTION)
				{
					mIgnoreCertificateErrors.setSelected(true);
					okPressed();
					return;
				}
			}
			
			ex.printStackTrace();
			JOptionPane.showMessageDialog(this, ex.getMessage(), getTitle() + " - Error", JOptionPane.ERROR_MESSAGE);
			return;
		}
		
		closeDialog();
	}

	protected Exchange getExchange()
	{
		return mExchange;
	}

	public static Exchange display()
	{
		try
		{
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		}
		catch(Exception e)
		{
		}

		ExchangeConnectionDialog dlg = new ExchangeConnectionDialog();
		dlg.setVisible(true);

		return dlg.getExchange();
	}
}
