import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.util.List;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import javax.swing.tree.*;

import com.moyosoft.exchange.*;
import com.moyosoft.exchange.calendar.*;
import com.moyosoft.exchange.contact.*;
import com.moyosoft.exchange.folder.*;
import com.moyosoft.exchange.item.*;
import com.moyosoft.exchange.mail.*;
import com.moyosoft.exchange.task.*;

public class ExchangeExplorer extends JFrame
{
	public static void main(String[] args)
	{
		try
		{
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		}
		catch(Exception e)
		{
		}

		ExchangeExplorer.open();
	}

	public static void open()
	{
		ExchangeExplorer frame = new ExchangeExplorer();
		frame.setVisible(true);
	}

	private JButton mButtonAdd = new JButton("Add new item");
	private JButton mButtonEdit = new JButton("Edit");
	private JButton mButtonRemove = new JButton("Remove");
	private FoldersPanel mFoldersPanel = new FoldersPanel();
	private JTable mTable = null;
	private ExchangeTableModel mTableModel = null;

	private Exchange exchange = null;

	public ExchangeExplorer()
	{
		super("Java Bridge to Exchange - Demo application");
		init();
	}

	public ExchangeExplorer(String title)
	{
		super(title);
		init();
	}

	private void init()
	{
		setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);

		addWindowListener(new java.awt.event.WindowAdapter()
		{
			public void windowClosing(java.awt.event.WindowEvent e)
			{
				exitApplication();
			};

			public void windowOpened(WindowEvent e)
			{
				initApplication();
			}
		});

		mFoldersPanel.setPreferredSize(new Dimension(150, 100));
		mFoldersPanel.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 4));
		
		JPanel foldersPanel = createFoldersPanel();
		JPanel buttonsPanel = createButtonsPanel();
		JPanel tablePanel = createTablePanel();

		JPanel mainPanel = new JPanel(new BorderLayout());
		mainPanel.setBorder(BorderFactory.createEmptyBorder(4, 4, 4, 4));
		mainPanel.add(foldersPanel, BorderLayout.WEST);
		mainPanel.add(tablePanel, BorderLayout.CENTER);
		mainPanel.add(buttonsPanel, BorderLayout.SOUTH);

		this.getContentPane().setLayout(new BorderLayout());
		this.getContentPane().add(mainPanel, BorderLayout.CENTER);

		setSize(620, 420);
		centerOnScreen(this);
	}

	private static void centerOnScreen(Window window)
	{
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension windowSize = window.getSize();
		windowSize.height = Math.min(windowSize.height + 20, screenSize.height);
		windowSize.width = Math.min(windowSize.width, screenSize.width);
		window.setLocation((screenSize.width - windowSize.width) / 2,
				(screenSize.height - windowSize.height) / 2);
	}

	private static void setWaitCursorSafe(Container pContainer)
	{
		try
		{
			pContainer.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		}
		catch(Throwable t)
		{
		}
	}

	private static void setDefaultCursorSafe(Container pContainer)
	{
		try
		{
			pContainer.setCursor(Cursor.getDefaultCursor());
		}
		catch(Throwable t)
		{
		}
	}

	private JPanel createFoldersPanel()
	{
		JPanel panel = new JPanel(new BorderLayout());
		
		JLabel label = new JLabel("Folders");
		label.setBorder(
				BorderFactory.createCompoundBorder(
						BorderFactory.createEmptyBorder(0, 0, 4, 4),
						BorderFactory.createCompoundBorder(
							BorderFactory.createEtchedBorder(),
							BorderFactory.createEmptyBorder(4, 4, 4, 4)
						)
					)
				);

		panel.add(label, BorderLayout.NORTH);
		panel.add(mFoldersPanel, BorderLayout.CENTER);
		
		mFoldersPanel.mMainTree.addTreeSelectionListener(
			new TreeSelectionListener()
			{
				public void valueChanged(TreeSelectionEvent e)
				{
					treeSelectionChanged();
				}
			});
		
		return panel;
	}
	
	private JPanel createButtonsPanel()
	{
		JPanel panel = new JPanel(new FlowLayout(FlowLayout.LEFT));

		mButtonAdd.setEnabled(false);
		mButtonEdit.setEnabled(false);
		mButtonRemove.setEnabled(false);

		panel.add(mButtonAdd);
		panel.add(mButtonEdit);
		panel.add(mButtonRemove);

		mButtonEdit.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				editItem();
			}
		});
		mButtonAdd.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				addItem();
			}
		});
		mButtonRemove.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				removeItem();
			}
		});

		return panel;
	}

	private JPanel createTablePanel()
	{
		JPanel panel = new JPanel(new BorderLayout());

		mTableModel = new ExchangeTableModel();
		mTable = new JTable(mTableModel);

		JScrollPane scrollPane = new JScrollPane(mTable,
				JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scrollPane.getViewport().setBackground(Color.white);

		panel.add(scrollPane, BorderLayout.CENTER);

		mTable.getSelectionModel().addListSelectionListener(
				new ListSelectionListener()
				{
					public void valueChanged(ListSelectionEvent arg0)
					{
						tableSelectionChanged();
					}
				});
		
		return panel;
	}

	private void treeSelectionChanged()
	{
//		Thread t = new Thread(new Runnable()
//		{
//			public void run()
//			{
				setWaitCursorSafe(getContentPane());
				mButtonAdd.setEnabled(false);
				mButtonEdit.setEnabled(false);
				mButtonRemove.setEnabled(false);
				try
				{
					ExchangeFolder folder = mFoldersPanel.getSelectedFolder();
					mTableModel.showFolder(folder);
					
					boolean enableButtons = 
						folder != null &&
						(folder.getContentType() == FolderContentType.Contacts ||
						folder.getContentType() == FolderContentType.Mails);
					
					mButtonAdd.setEnabled(enableButtons);
					mButtonEdit.setEnabled(enableButtons);
					mButtonRemove.setEnabled(enableButtons);
				}
				catch(ExchangeServiceException ex)
				{
					showError(ex);
				}
				finally
				{
					setDefaultCursorSafe(getContentPane());
				}
//			}
//		});
//		
//		t.start();
	}

	private void tableSelectionChanged()
	{
	}

	private void showError(ExchangeServiceException ex)
	{
		ex.printStackTrace();
		
		JOptionPane.showMessageDialog(
				this,
				ex.getMessage(),
				"Error",
				JOptionPane.ERROR_MESSAGE
				);
	}

	private void setFolder(final ExchangeFolder folder)
	{
		if(folder == null)
		{
			return;
		}

		Thread t = new Thread(new Runnable()
		{
			public void run()
			{
				setWaitCursorSafe(ExchangeExplorer.this.getContentPane());
				try
				{
					mFoldersPanel.setRootFolder(folder);
				}
				catch(ExchangeServiceException ex)
				{
					showError(ex);
				}
				finally
				{
					setDefaultCursorSafe(ExchangeExplorer.this.getContentPane());
					tableSelectionChanged();
				}
			}
		});
		
		t.start();
	}

	private void refresh()
	{
		treeSelectionChanged();
	}
	
	private void addItem()
	{
		try
		{
			ExchangeFolder folder = mFoldersPanel.getSelectedFolder();
			
			if(folder != null)
			{
				switch(folder.getContentType())
				{
				case Mails:
					ExchangeMailDialog.open(this, folder.createMail(), true);
					break;
					
				case Contacts:
					ExchangeContactDialog.open(this, folder.createContact(), true);
					break;
				}
				
				refresh();
			}
		}
		catch(ExchangeServiceException ex)
		{
			showError(ex);
		}
	}
	
	private void editItem()
	{
		int selectedRow = mTable.getSelectedRow();
		
		if(selectedRow >= 0)
		{
			try
			{
				ExchangeItem item = mTableModel.getItemAt(selectedRow);
				
				if(item != null)
				{
					if(item instanceof ExchangeMail)
					{
						ExchangeMailDialog.open(this, (ExchangeMail) item, false);
					}
					else if(item instanceof ExchangeContact)
					{
						ExchangeContactDialog.open(this, (ExchangeContact) item, false);
					}
					
					refresh();
				}
			}
			catch(ExchangeServiceException ex)
			{
				showError(ex);
			}
		}
	}
	
	private void removeItem()
	{
		int selectedRow = mTable.getSelectedRow();
		
		if(selectedRow >= 0)
		{
			try
			{
				ExchangeItem item = mTableModel.getItemAt(selectedRow);
				
				if(item != null)
				{
					item.delete(true);
					refresh();
				}
			}
			catch(ExchangeServiceException ex)
			{
				showError(ex);
			}
		}
	}
	
	private void initApplication()
	{
		try
		{
			exchange = ExchangeConnectionDialog.display();

			if(exchange == null)
			{
				exitApplication();
				return;
			}
			
			setFolder(exchange.getTopFolder());
		}
		catch(ExchangeServiceException ex)
		{
			showError(ex);
		}
	}

	private void exitApplication()
	{
		dispose();
	}

	class FoldersPanel extends JPanel implements TreeSelectionListener, TreeWillExpandListener
	{
		private JTree mMainTree = new JTree();

		protected FoldersPanel()
		{
			super();
			initialize();
		}

		private void initialize()
		{
			this.setLayout(new java.awt.BorderLayout());
			this.setBorder(BorderFactory.createEmptyBorder(5, 5, 0, 5));

			DefaultTreeCellRenderer renderer = new DefaultTreeCellRenderer();
			renderer.setLeafIcon(renderer.getClosedIcon());

			mMainTree.setShowsRootHandles(true);
			mMainTree.setRootVisible(false);
			mMainTree.setCellRenderer(renderer);

			JScrollPane scrollPanel = new JScrollPane();
			scrollPanel.setViewportView(mMainTree);

			this.add(scrollPanel, BorderLayout.CENTER);

			mMainTree.setModel(new DefaultTreeModel(new FolderTreeNode()));

			mMainTree.addTreeWillExpandListener(this);
			mMainTree.addTreeSelectionListener(this);
		}

		public void setRootFolder(ExchangeFolder topFolder) throws ExchangeServiceException
		{
			FolderTreeNode rootNode = new FolderTreeNode(topFolder);
			rootNode.createChildrens();

			mMainTree.setModel(new DefaultTreeModel(rootNode));
			mMainTree.expandPath(new TreePath(new Object[] { rootNode }));
		}
		
		protected JTree getTree()
		{
			return mMainTree;
		}

		protected ExchangeFolder getSelectedFolder()
		{
			return getFolderForPath(mMainTree.getSelectionPath());
		}

		protected ExchangeFolder getFolderForPath(TreePath pPath)
		{
			if(pPath == null)
			{
				return null;
			}

			return ((FolderTreeNode) (pPath.getLastPathComponent())).getFolder();
		}

		protected void expandSelectedItem()
		{
			TreePath path = mMainTree.getSelectionPath();

			if(path != null)
			{
				mMainTree.expandPath(path);
			}
		}

		protected void collapseSelectedItem()
		{
			TreePath path = mMainTree.getSelectionPath();

			if(path != null)
			{
				mMainTree.collapsePath(path);
			}
		}

		public void treeWillCollapse(TreeExpansionEvent event)
				throws ExpandVetoException
		{
		}

		public void treeWillExpand(TreeExpansionEvent event)
				throws ExpandVetoException
		{
			if(event.getSource() == mMainTree)
			{
				Object o = event.getPath().getLastPathComponent();

				if(o instanceof FolderTreeNode)
				{
					try
					{
					((FolderTreeNode) o).createChildrens();
					}
					catch(ExchangeServiceException ex)
					{
						showError(ex);
					}
				}
			}
		}

		public void valueChanged(TreeSelectionEvent e)
		{
		}
	}
	
	static class FolderTreeNode extends DefaultMutableTreeNode
	{
		static class FolderComparator implements Comparator<ExchangeFolder>
		{
		    public int compare(ExchangeFolder folder1, ExchangeFolder folder2)
		    {
		    	try
		    	{
			        if(folder1.getDisplayName() == null)
			        {
			            return -1;
			        }
		    	}
		    	catch(ExchangeServiceException ex)
		    	{
		    		return -1;
		    	}
		        
		    	try
		    	{
			        if(folder2.getDisplayName() == null)
			        {
			            return 1;
			        }
			        
			        return folder1.getDisplayName().compareToIgnoreCase(folder2.getDisplayName());
		    	}
		    	catch(ExchangeServiceException ex)
		    	{
		    		return 1;
		    	}
		    }
		}
		
	    private ExchangeFolder mFolder;

	    public FolderTreeNode()
	    {
	        super("", true);
	    }

	    public FolderTreeNode(ExchangeFolder folder) throws ExchangeServiceException
	    {
	        super(folder.getDisplayName(), true);
	        mFolder = folder;
	    }
		
		private void createChildrens(FoldersCollection folders) throws ExchangeServiceException
		{
			if(folders != null && folders.getCount() > 0)
			{
			    List<ExchangeFolder> foldersList = new ArrayList<ExchangeFolder>();
			    
				for(FoldersIterator it = folders.iterator(); it.hasNext();)
				{
					ExchangeFolder folder = it.nextFolder();
					foldersList.add(folder);
				}
			    
				Collections.sort(foldersList, new FolderComparator());

				for(ExchangeFolder folder : foldersList)
				{
					FolderTreeNode node = new FolderTreeNode(folder);
					add(node);
				}
			}
		}

	    public void createChildrens() throws ExchangeServiceException
	    {
	        if(mFolder != null && mFolder.getFoldersCount() > 0)
	        {
	            createChildrens(mFolder.getFolders());
	        }
	    }

	    public ExchangeFolder getFolder()
	    {
	        return mFolder;
	    }

	    public boolean isLeaf()
	    {
	    	try
	    	{
		        if(mFolder != null)
		        {
		            return mFolder.getFoldersCount() <= 0;
		        }
	    	}
	    	catch(ExchangeServiceException ex)
	    	{
	    	}

	        return false;
	    }
	}

	static class ExchangeTableModel extends AbstractTableModel
	{
		private static final String[] MessageHeaders 	= new String[] { "From", "Subject", "When", "Size" };
		private static final String[] ContactHeaders 	= new String[] { "Name", "Company", "Phone", "E-mail" };
		private static final String[] CalendarHeaders 	= new String[] { "Subject", "Start", "End", "Location" };
		private static final String[] TaskHeaders 		= new String[] { "Subject", "Due by" };

		private ExchangeItem[] mItems;
		private ItemsCollection mItemsCollection;
		private String[] mHeaders = MessageHeaders;
		
		public ExchangeTableModel()
		{
		}

		public synchronized void showFolder(ExchangeFolder folder) throws ExchangeServiceException
		{
			if(folder != null)
			{
				ItemsCollection items = folder.getItems();
				FolderContentType contentType = folder.getContentType();
				
				if(contentType != null)
				{
					switch(contentType)
					{
					case Calendar:
						mHeaders = CalendarHeaders;
						items = items.sortBy(ExchangeItemField.calendar_Start, false);
						break;
						
					case Contacts:
						mHeaders = ContactHeaders;
						items = items.sortBy(ExchangeItemField.contact_DisplayName, false);
						break;
						
					case Tasks: 		mHeaders = TaskHeaders; break;
					case SearchFolder: 	mHeaders = MessageHeaders; break;
					
					case Mails:
						mHeaders = MessageHeaders;
						items = items.sortBy(ExchangeItemField.item_DateTimeReceived, false);
						break;
						
					default:	 		mHeaders = null; break;
					}
				}

				mItemsCollection = items;
				mItems = new ExchangeItem[mItemsCollection.getCount()];
				getValueAt(0, 0);
			}
			else
			{
				mItemsCollection = null;
				mItems = null;
			}

			fireTableStructureChanged();
		}

		public synchronized String getColumnName(int column)
		{
			if(mHeaders == null)
			{
				return "";
			}
			
			return mHeaders[column];
		}

		public synchronized int getRowCount()
		{
			if(mItems == null)
			{
				return 0;
			}

			return mItems.length;
		}

		public synchronized int getColumnCount()
		{
			if(mHeaders == null)
			{
				return 0;
			}
			
			return mHeaders.length;
		}

		public synchronized ExchangeItem getItemAt(int row)
		{
			if(mItems == null || row >= mItems.length)
			{
				return null;
			}

			if(mItems[row] == null)
			{
				try
				{
					if(mItemsCollection != null)
					{
						int i = row - 20;
						
						if(i < 0)
						{
							i = 0;
						}
						
						for(ExchangeItem loadedItem : mItemsCollection.iterableFromIndex(i, 60))
						{
							if(i < mItems.length && mItems[i] == null)
							{
								mItems[i] = loadedItem;
							}
							
							i++;
						}
					}
				}
				catch(ExchangeServiceException ex)
				{
					ex.printStackTrace();
				}
			}
			
			return mItems[row];
		}
		
		public synchronized Object getValueAt(int row, int column)
		{
			if(mItems == null)
			{
				return "";
			}
			
			try
			{
				ExchangeItem item = getItemAt(row);
				
				if(item != null)
				{
					if(item instanceof ExchangeMail)
					{
						ExchangeMail mail = (ExchangeMail) item;
						
						switch(column)
						{
						case 0: return mail.getFromName();
						case 1: return mail.getSubject();
						case 2: return mail.getDateTimeCreated();
						case 3: return mail.getSize();
						}
					}
					else if(item instanceof ExchangeContact)
					{
						ExchangeContact contact = (ExchangeContact) item;
						
						switch(column)
						{
						case 0: return contact.getDisplayName();
						case 1: return contact.getCompanyName();
						case 2: return contact.getPhoneNumberHomePhone();
						case 3: return contact.getEmailAddress1();
						}
					}
					else if(item instanceof ExchangeCalendarItem)
					{
						ExchangeCalendarItem calendarItem = (ExchangeCalendarItem) item;
						
						switch(column)
						{
						case 0: return calendarItem.getSubject();
						case 1: return calendarItem.getStart();
						case 2: return calendarItem.getEnd();
						case 3: return calendarItem.getLocation();
						}
					}
					else if(item instanceof ExchangeTask)
					{
						ExchangeTask task = (ExchangeTask) item;
						
						switch(column)
						{
						case 0: return task.getSubject();
						case 1: return task.getDueDate();
						}
					}
				}
			}
			catch(ExchangeServiceException ex)
			{
				ex.printStackTrace();
				return "";
			}

			return "";
		}
	}
}
