import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.util.List;

import javax.swing.*;

import com.moyosoft.exchange.*;
import com.moyosoft.exchange.mail.*;

public class ExchangeMailDialog extends JDialog
{
	private ExchangeMail mMail;
	private MailPanel mMailPanel = new MailPanel(this);
	private boolean mAddingNew;
	
	public ExchangeMailDialog(Frame parent, boolean addingNew)
	{
		super(parent);
		mAddingNew = addingNew;
		init();
	}
	
	protected void init()
	{
		setModal(true);
		setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(BorderLayout.CENTER, mMailPanel);
	}
	
	public void close()
	{
		dispose();
	}
	
	public void load(ExchangeMail mail) throws ExchangeServiceException
	{
		mMail = mail;
		
		if(mMail != null)
		{
			mMailPanel.load(mMail);
		}
		
		mailChanged();
	}
	
	protected void mailChanged() throws ExchangeServiceException
	{
		updateTitle();
	}
	
	private static String notNull(String str)
	{
		if(str == null) return "";
		return str;
	}
	
	protected void updateTitle() throws ExchangeServiceException
	{
		if(mMail != null)
		{
			setTitle("Mail - " + notNull(mMail.getSubject()));
		}
		else
		{
			setTitle("Mail");
		}
	}

	private void showError(ExchangeServiceException ex)
	{
		ex.printStackTrace();
		
		JOptionPane.showMessageDialog(
				this,
				ex.getMessage(),
				"Error",
				JOptionPane.ERROR_MESSAGE
				);
	}
	
	public void closePressed()
	{
		if(mAddingNew && mMail != null)
		{
			try
			{
				mMail.delete(false);
			}
			catch(ExchangeServiceException ex)
			{
			}
		}
		
		close();
	}
	
	public void savePressed()
	{
		try
		{
			mMailPanel.store(mMail);
			mMail.save();
			
			close();
		}
		catch(ExchangeServiceException ex)
		{
			showError(ex);
		}
	}
	
	private static void centerOnScreen(Window window)
	{
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension windowSize = window.getSize();
		windowSize.height = Math.min(windowSize.height + 20, screenSize.height);
		windowSize.width = Math.min(windowSize.width, screenSize.width);
		window.setLocation((screenSize.width - windowSize.width) / 2,
				(screenSize.height - windowSize.height) / 2);
	}
	
	public static void open(Frame parent, ExchangeMail mail, boolean addingNew) throws ExchangeServiceException
	{
		ExchangeMailDialog dialog = new ExchangeMailDialog(parent, addingNew);
		dialog.pack();
		centerOnScreen(dialog);
		
		dialog.load(mail);
		
		dialog.setVisible(true);
	}
}

class MailPanel extends JPanel
{
	private ExchangeMailDialog mParent;
	private JButton mButtonClose = new JButton("Cancel");
	private JButton mButtonSave = new JButton("Save");
	private MailAttributesPanel mMailAttributesPanel = new MailAttributesPanel();
	
	public MailPanel(ExchangeMailDialog parent)
	{
		super();
		mParent = parent;
		init();
	}
	
	protected void init()
	{
		createPanel();
	}
	
	protected void createPanel()
	{
		// boutons
		JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
		buttonPanel.add(mButtonSave);
		buttonPanel.add(mButtonClose);
		
		setLayout(new BorderLayout());
		add(BorderLayout.CENTER, mMailAttributesPanel);
		add(BorderLayout.SOUTH, buttonPanel);
		
		// borders
		mMailAttributesPanel.setBorder(BorderFactory.createEmptyBorder(14, 7, 5, 7));
		buttonPanel.setBorder(BorderFactory.createEmptyBorder(0, 4, 4, 4));
		
		// listeners
		mButtonClose.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event)
			{
				closePressed();
			}
		});
		mButtonSave.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event)
			{
				savePressed();
			}
		});
	}
	
	public void closePressed()
	{
		mParent.closePressed();
	}
	
	public void savePressed()
	{
		mParent.savePressed();
	}
	
	public void load(ExchangeMail mail) throws ExchangeServiceException
	{
		mMailAttributesPanel.load(mail);
	}
	
	public void store(ExchangeMail mail) throws ExchangeServiceException
	{
		mMailAttributesPanel.store(mail);
	}
}

class MailAttributesPanel extends JPanel
{
	private JTextField mFieldFrom = new JTextField();
	private JTextField mFieldTo = new JTextField();
	private JTextField mFieldSubject = new JTextField();
	private ExchangeAttachmentsPanel mFieldAttachments = new ExchangeAttachmentsPanel();
	private JTextArea mFieldBody = new JTextArea();
	
	public MailAttributesPanel()
	{
		super();
		init();
	}
	
	public MailAttributesPanel(boolean pDoubleBuffered)
	{
		super(pDoubleBuffered);
		init();
	}
	
	protected void init()
	{
		createPanel();
		mFieldFrom.setEditable(false);
		
		setPreferredSize(new Dimension(580, 360));
	}
	
	protected void createPanel()
	{
		// Create panels
		JPanel panelInfo = createInfoPanel();
		JPanel panelBody = createBodyPanel();
		JPanel panelAttachments = createAttachmentsPanel();
		
		JLabel separator1 = new JLabel("");
		JLabel separator2 = new JLabel("");
		JLabel separator3 = new JLabel("");
		
		// GridBagLayout
		setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		
		// insets between components
		Insets separatorInsets = new Insets(10, 3, 0, 3);
		Insets leftInsets = new Insets(0, 3, 0, 10);
		
		// two columns: 50% of the width each
		c.fill = GridBagConstraints.HORIZONTAL;
		c.anchor = GridBagConstraints.NORTHWEST;
		c.weightx = 1;
		
		c.gridx = 0;
		c.gridy = 0;
		c.insets = leftInsets;
		add(panelInfo, c);
		
		c.gridx = 0;
		c.gridy = 1;
		c.insets = separatorInsets;
		add(separator1, c);
		
		c.fill = GridBagConstraints.BOTH;
		c.gridx = 0;
		c.gridy = 2;
		c.weighty = 1;
		c.insets = leftInsets;
		add(panelBody, c);
		
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weighty = 0;
		c.gridx = 0;
		c.gridy = 3;
		c.insets = new Insets(10, 3, 0, 3);
		add(separator2, c);
		
		c.gridx = 0;
		c.gridy = 4;
		c.insets = leftInsets;
		add(new JLabel("Attachments"), c);
		
		c.gridx = 0;
		c.gridy = 5;
		c.insets = leftInsets;
		add(panelAttachments, c);
		
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weighty = 0;
		c.gridx = 0;
		c.gridy = 6;
		c.insets = separatorInsets;
		add(separator3, c);
	}
	
	private JPanel createInfoPanel()
	{
		JPanel panel = new JPanel(new GridBagLayout());
		
		GridBagConstraints labelConstraints = new GridBagConstraints();
		labelConstraints.insets = new Insets(0, 0, 0, 7);
		labelConstraints.anchor = GridBagConstraints.WEST;
		
		GridBagConstraints fieldConstraints = new GridBagConstraints();
		fieldConstraints.fill = GridBagConstraints.HORIZONTAL;
		fieldConstraints.insets = new Insets(0, 0, 2, 0);
		fieldConstraints.weightx = 1;
		
		// 0,0
		labelConstraints.gridx = 0;
		labelConstraints.gridy = 0;
		panel.add(new JLabel("From"), labelConstraints);
		
		// 1,0
		fieldConstraints.gridx = 1;
		fieldConstraints.gridy = 0;
		panel.add(mFieldFrom, fieldConstraints);
		
		// 0,0
		labelConstraints.gridx = 0;
		labelConstraints.gridy = 1;
		panel.add(new JLabel("To"), labelConstraints);
		
		// 1,0
		fieldConstraints.gridx = 1;
		fieldConstraints.gridy = 1;
		panel.add(mFieldTo, fieldConstraints);
		
		// 0,1
		labelConstraints.gridx = 0;
		labelConstraints.gridy = 2;
		panel.add(new JLabel("Subject"), labelConstraints);
		
		// 1,1
		fieldConstraints.gridx = 1;
		fieldConstraints.gridy = 2;
		panel.add(mFieldSubject, fieldConstraints);
		
		return panel;
	}
	
	private JPanel createBodyPanel()
	{
		JPanel panel = new JPanel(new BorderLayout());
		
		panel.add(BorderLayout.NORTH, new JLabel("Body"));
		
		JScrollPane scrollPane = new JScrollPane(mFieldBody);
		panel.add(BorderLayout.CENTER, scrollPane);
		
		panel.setPreferredSize(new Dimension(10, 10));
		
		return panel;
	}
	
	private JPanel createAttachmentsPanel()
	{
		JPanel panel = new JPanel(new BorderLayout());
		
		panel.add(BorderLayout.CENTER, mFieldAttachments);
		panel.setPreferredSize(new Dimension(10, 80));
		
		return panel;
	}

	private static String createRecipientsText(ExchangeMail mail) throws ExchangeServiceException
	{
		List<ExchangeMailbox> mailboxes = mail.getToRecipientsMailboxes();
		
		if(mailboxes == null || mailboxes.size() == 0)
		{
			return "";
		}

		StringBuilder builder = new StringBuilder("");
		
		for(ExchangeMailbox mailbox : mailboxes)
		{
			builder.append(mailbox.getEmailAddress());
			builder.append("; ");
		}
		
		return builder.toString().trim();
	}
	
	public void load(ExchangeMail mail) throws ExchangeServiceException
	{
		if(mail == null)
		{
			return;
		}

		mFieldFrom.setText(mail.getFromEmailAddress());
		mFieldTo.setText(createRecipientsText(mail));
		mFieldSubject.setText(mail.getSubject());
		mFieldBody.setText(mail.getBody());
		mFieldAttachments.load(mail);
	}
	
	private List<String> createRecipientsList()
	{
		String to = mFieldTo.getText();
		
		if(to == null || to.trim().length() == 0)
		{
			return null;
		}
		
		StringTokenizer tokenizer = new StringTokenizer(to, ";,");
		ArrayList<String> list = new ArrayList<String>();
		
		while(tokenizer.hasMoreTokens())
		{
			String toAddress = tokenizer.nextToken();
			
			if(toAddress != null && toAddress.trim().length() > 0)
			{
				list.add(toAddress);
			}
		}
		
		return list;
	}
	
	public void store(ExchangeMail mail) throws ExchangeServiceException
	{
		if(!mFieldTo.getText().equals(createRecipientsText(mail)))
		{
			mail.setToRecipients(createRecipientsList());
		}

		if(!mFieldSubject.getText().equals(mail.getSubject()))
		{
			mail.setSubject(mFieldSubject.getText());
		}
		
		if(!mFieldBody.getText().equals(mail.getBody()))
		{
			mail.setBody(mFieldBody.getText());
		}
	}
}
