package com.moyosoft.exchange.javamail;

import java.io.*;

import javax.mail.*;
import javax.mail.internet.*;

import com.moyosoft.exchange.*;
import com.moyosoft.exchange.mail.*;
import com.sun.mail.util.*;

public class JbexTransport extends Transport
{
	private Exchange exchange;

	public JbexTransport(Session session, URLName urlname)
	{
		super(session, urlname);
	}

	protected boolean protocolConnect(String host, int port, String user, String password) throws MessagingException
	{
		try
		{
			exchange = new Exchange(host, user, password);
		}
		catch(ExchangeServiceException e)
		{
			handleError(e);
		}

		return true;
	}

	public void sendMessage(Message msg, Address[] addresses) throws MessagingException
	{
		try
		{
			ExchangeMail mail = createMail(msg, addresses);

			if(mail == null)
			{
				throw new MessagingException("Unable to create the message");
			}

			mail.send();
		}
		catch(ExchangeServiceException e)
		{
			handleError(e);
		}
	}

	private ExchangeMail createMail(Message msg, Address[] addresses) throws ExchangeServiceException, MessagingException
	{
		if(!(msg instanceof MimeMessage))
		{
			return null;
		}

		MimeMessage mimeMessage = (MimeMessage) msg;
		ByteArrayOutputStream mimeBytesStream = new ByteArrayOutputStream();

		try
		{
			mimeMessage.writeTo(new CRLFOutputStream(mimeBytesStream));
		}
		catch(IOException e)
		{
			throw new MessagingException("Unable to write message content", e);
		}

		byte[] mimeContent = mimeBytesStream.toByteArray();

		if(mimeContent == null || mimeContent.length == 0)
		{
			return null;
		}

		ExchangeMail mail = exchange.createMail();
		mail.setMimeContent(mimeContent);

		return mail;
	}

	private void handleError(ExchangeServiceException e) throws MessagingException
	{
		throw new MessagingException(e.toString(), e);
	}
}
